unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  SynHighlighterPas, SynEdit, USource;

type

  { TFormMain }

  TFormMain = class(TForm)
    ButtonCompile: TButton;
    ButtonRun: TButton;
    Label1: TLabel;
    Label2: TLabel;
    MemoLog: TMemo;
    MemoOutput: TMemo;
    SynEditSource: TSynEdit;
    SynFreePascalSyn1: TSynFreePascalSyn;
    procedure ButtonCompileClick(Sender: TObject);
    procedure ButtonRunClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    Prog: TProgram;
    Initialized: Boolean;
    procedure ExecutorOutput(Text: string);
    procedure InterpreterError(Pos: TPoint; Text: string);
    procedure UpdateInterface;
  public

  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UParser, UExecutor;

{ TFormMain }

procedure TFormMain.FormActivate(Sender: TObject);
begin
  if not Initialized then begin
    Initialized := True;
    SynEditSource.Lines.LoadFromFile('Test.pas');
    ButtonRun.Click;
  end;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin

end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  if Assigned(Prog) then Prog.Free;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormMain.ButtonRunClick(Sender: TObject);
var
  Executor: TExecutor;
begin
  ButtonCompile.Click;
  MemoOutput.Lines.Clear;
  if Assigned(Prog) then begin
    Executor := TExecutor.Create;
    Executor.Prog := Prog;
    Executor.OnOutput := ExecutorOutput;
    Executor.Run;
    Executor.Free;
  end;
end;

procedure TFormMain.ButtonCompileClick(Sender: TObject);
var
  Parser: TParser;
begin
  MemoLog.Lines.Clear;
  Parser := TParser.Create;
  Parser.OnError := InterpreterError;
  Parser.Source := SynEditSource.Lines.Text;
  Parser.Parse;
  if Assigned(Prog) then Prog.Free;
  Prog := Parser.Prog;
  Parser.Free;
  UpdateInterface;
end;

procedure TFormMain.InterpreterError(Pos: TPoint; Text: string);
begin
  MemoLog.Lines.Add('[' + IntToStr(Pos.X) + ', ' + IntToStr(Pos.Y) + '] ' + Text);
end;

procedure TFormMain.UpdateInterface;
begin
  ButtonRun.Enabled := Assigned(Prog);
end;

procedure TFormMain.ExecutorOutput(Text: string);
begin
  MemoOutput.Text := MemoOutput.Text + Text;
end;

end.

